@@ -72,8 +72,14 @@ struct _GnomeControlCenterPrivate
 
   gchar *default_window_title;
   gchar *default_window_icon;
+
+  gint overview_height;
 };
 
+/* Use a fixed width for the shell, since resizing horizontally is more awkward
+ * for the user than resizing vertically */
+#define FIXED_WIDTH 675
+
 
 static void
 activate_panel (GnomeControlCenter *shell,
@@ -119,7 +125,8 @@ activate_panel (GnomeControlCenter *shell,
         {
           GtkWidget *panel;
           GtkWidget *box;
-          gint i;
+          gint i, old_page;
+          GtkRequisition min, nat;
 
           /* create the panel plugin */
           panel = g_object_new (panel_type, "shell", shell, NULL);
@@ -129,9 +136,21 @@ activate_panel (GnomeControlCenter *shell,
 
           gtk_container_add (GTK_CONTAINER (box), panel);
 
-          /* switch to the new panel */
+          old_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+
+          /* store the old size */
+          if (old_page == OVERVIEW_PAGE)
+            {
+              GtkAllocation alloc;
+
+              gtk_widget_get_allocation (priv->window, &alloc);
+              priv->overview_height = alloc.height;
+            }
+
           i = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), box,
                                         NULL);
+
+          /* switch to the new panel */
           gtk_widget_show (box);
           gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), i);
 
@@ -140,6 +159,13 @@ activate_panel (GnomeControlCenter *shell,
           gtk_window_set_icon_name (GTK_WINDOW (priv->window), icon_name);
 
           gtk_widget_show (panel);
+
+          /* resize to the preferred size of the panel */
+          gtk_widget_set_size_request (priv->window, FIXED_WIDTH, -1);
+          gtk_size_request_get_size (GTK_SIZE_REQUEST (priv->window), &min,
+                                     &nat);
+          gtk_window_resize (GTK_WINDOW (priv->window), FIXED_WIDTH,
+                             nat.height);
           return;
         }
     }
@@ -200,6 +226,17 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
   gtk_window_set_title (GTK_WINDOW (priv->window), priv->default_window_title);
   gtk_window_set_icon_name (GTK_WINDOW (priv->window),
                             priv->default_window_icon);
+
+  /* resize back to the original overview height */
+  if (priv->overview_height > 0)
+    {
+      gtk_widget_set_size_request (priv->window,
+                                   FIXED_WIDTH,
+                                   priv->overview_height);
+      gtk_window_resize (GTK_WINDOW (priv->window),
+                         FIXED_WIDTH,
+                         priv->overview_height);
+    }
 }
 
 
@@ -460,7 +497,7 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
                                  GdkEventKey *event,
                                  GnomeControlCenterPrivate   *priv)
 {
-  if (event->keyval == GDK_Return)
+  if (event->keyval == GDK_KEY_Return)
     {
       GtkTreePath *path;
 
@@ -474,7 +511,7 @@ search_entry_key_press_event_cb (GtkEntry    *entry,
       return TRUE;
     }
 
-  if (event->keyval == GDK_Escape)
+  if (event->keyval == GDK_KEY_Escape)
     {
       gtk_entry_set_text (entry, "");
       return TRUE;
@@ -509,8 +546,8 @@ setup_search (GnomeControlCenter *shell)
 
   /* set up the search view */
   priv->search_view = search_view = cc_shell_item_view_new ();
-  gtk_icon_view_set_orientation (GTK_ICON_VIEW (search_view),
-                                 GTK_ORIENTATION_HORIZONTAL);
+  gtk_icon_view_set_item_orientation (GTK_ICON_VIEW (search_view),
+                                      GTK_ORIENTATION_HORIZONTAL);
   gtk_icon_view_set_spacing (GTK_ICON_VIEW (search_view), 6);
   gtk_icon_view_set_model (GTK_ICON_VIEW (search_view),
                            GTK_TREE_MODEL (priv->search_filter));
@@ -627,6 +664,8 @@ fill_model (GnomeControlCenter *shell)
                                            f->data);
                 }
             }
+
+          g_slist_free (contents);
         }
     }
 
@@ -668,7 +707,7 @@ home_button_clicked_cb (GtkButton *button,
 
 static void
 notebook_switch_page_cb (GtkNotebook               *book,
-                         GtkNotebookPage           *page,
+                         GtkWidget                 *child,
                          gint                       page_num,
                          GnomeControlCenterPrivate *priv)
 {
@@ -863,13 +902,28 @@ gnome_control_center_class_init (GnomeControlCenterClass *klass)
   shell_class->set_active_panel_from_id = _shell_set_active_panel_from_id;
 }
 
+static void
+vbox_style_set_cb (GtkWidget *widget,
+                   GtkStyle  *old_style)
+{
+  GtkWidget *parent;
+  GtkStyle *style;
+
+  parent = gtk_widget_get_parent (widget);
+  style = gtk_widget_get_style (widget);
+  gtk_widget_modify_bg (parent, GTK_STATE_NORMAL,
+                        &style->base[GTK_STATE_NORMAL]);
+  gtk_widget_modify_fg (parent, GTK_STATE_NORMAL,
+                        &style->text[GTK_STATE_NORMAL]);
+
+}
+
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
   GError *err = NULL;
-  GtkWidget *vbox, *parent;
+  GtkWidget *vbox;
   GnomeControlCenterPrivate *priv;
-  GtkStyle *style;
 
   priv = self->priv = CONTROL_CENTER_PRIVATE (self);
 
@@ -902,12 +956,11 @@ gnome_control_center_init (GnomeControlCenter *self)
   vbox = W (priv->builder, "main-vbox");
   gtk_widget_set_size_request (vbox, 0, -1);
 
-  parent = gtk_widget_get_parent (vbox);
-  style = gtk_widget_get_style (vbox);
-  gtk_widget_modify_bg (parent, GTK_STATE_NORMAL,
-                        &style->base[GTK_STATE_NORMAL]);
-  gtk_widget_modify_fg (parent, GTK_STATE_NORMAL,
-                        &style->text[GTK_STATE_NORMAL]);
+  /* make sure the background of the vbox uses base/text colour combinations
+   * and is updated if the style changes */
+  vbox_style_set_cb (vbox, NULL);
+  g_signal_connect (vbox, "style-set", G_CALLBACK (vbox_style_set_cb),
+                    NULL);
 
   /* load the available settings panels */
   fill_model (self);
