@@ -483,7 +483,7 @@ postpone_entry_key_press_event_cb (GtkEntry       *entry,
 
 	priv = window->priv;
 
-	if (event->keyval == GDK_Escape) {
+	if (event->keyval == GDK_KEY_Escape) {
 		if (priv->postpone_timeout_id) {
 			g_source_remove (priv->postpone_timeout_id);
 		}
@@ -542,80 +542,36 @@ postpone_clicked_cb (GtkWidget *button,
 			  bw);
 }
 
-static void
-get_layout_location (GtkLabel *label,
-                     gint     *xp,
-                     gint     *yp)
-{
-	GtkMisc        *misc;
-	GtkWidget      *widget;
-	GtkAllocation  widget_allocation;
-	GtkRequisition widget_requisition;
-	gfloat         xalign, yalign;
-	gint           x, y;
-	gint           xpad, ypad;
-
-	misc = GTK_MISC (label);
-	widget = GTK_WIDGET (label);
-
-	gtk_misc_get_alignment (misc, &xalign, &yalign);
-	gtk_misc_get_padding (misc, &xpad, &ypad);
-	gtk_widget_get_allocation (widget, &widget_allocation);
-	gtk_widget_get_requisition (widget, &widget_requisition);
-
-	if (gtk_widget_get_direction (widget) != GTK_TEXT_DIR_LTR)
-		xalign = 1.0 - xalign;
-
-	x = floor (widget_allocation.x + (int)xpad
-		   + ((widget_allocation.width - widget_requisition.width - 1) * xalign)
-		   + 0.5);
-
-	y = floor (widget_allocation.y + (int)ypad
-		   + ((widget_allocation.height - widget_requisition.height - 1) * yalign)
-		   + 0.5);
-
-	if (xp) {
-		*xp = x;
-	}
-
-	if (yp) {
-		*yp = y;
-	}
-}
-
 static gboolean
 label_expose_event_cb (GtkLabel       *label,
 		       GdkEventExpose *event,
 		       gpointer        user_data)
 {
-	gint       x, y;
-	GdkColor   color;
-	GtkWidget *widget;
-	GdkWindow *window;
-	GdkGC     *gc;
+	gint             x, y;
+	GtkWidget       *widget;
+	GdkWindow       *window;
+        cairo_t         *cr;
 
-	color.red = 0;
-	color.green = 0;
-	color.blue = 0;
-	color.pixel = 0;
-
-	get_layout_location (label, &x, &y);
+        gtk_label_get_layout_offsets (label, &x, &y);
 
 	widget = GTK_WIDGET (label);
 	window = gtk_widget_get_window (widget);
 
-	gc = gdk_gc_new (window);
-	gdk_gc_set_rgb_fg_color (gc, &color);
-	gdk_gc_set_clip_rectangle (gc, &event->area);
+        cr = gdk_cairo_create (window);
 
-	gdk_draw_layout_with_colors (window,
-				     gc,
-				     x + 1,
-				     y + 1,
-				     gtk_label_get_layout (label),
-				     &color,
-				     NULL);
-	g_object_unref (gc);
+        gdk_cairo_rectangle (cr, &event->area);
+        cairo_clip (cr);
+
+        cairo_set_source_rgb (cr, 0, 0, 0);
+
+        /* Can't use pango_cairo_show_layout() here as we need to override
+         * the layout's colors with our shadow color.
+         */
+        cairo_move_to (cr, x + 1, y + 1);
+        pango_cairo_layout_path (cr, gtk_label_get_layout (label));
+        cairo_fill (cr);
+
+        cairo_destroy (cr);
 
 	gtk_paint_layout (gtk_widget_get_style (widget),
 			  window,
